use firefly_iii_api::apis::configuration::Configuration;
use firefly_iii_api::apis::{accounts_api, transactions_api};
use firefly_iii_api::models::{AccountRead, TransactionRead};

pub(super) async fn load_all_transactions(ff: &mut FFTransactions) -> Result<(), ()> {
    let mut has_more = true;
    let mut page = None;
    while has_more {
        match transactions_api::list_transaction(
            &ff.config,
            None,
            Some(500),
            page,
            None,
            None,
            None,
        )
        .await
        {
            Ok(transactions) => {
                has_more = transactions.links.next.is_some();
                let pagination = transactions.meta.pagination.clone().unwrap();
                let next = pagination.current_page.unwrap() + 1;
                page = Some(next);

                println!(
                    "Page {} of {}",
                    pagination.current_page.unwrap(),
                    pagination.total_pages.unwrap()
                );
                transactions
            }
            Err(e) => {
                dbg!(e);
                return Ok(());
            }
        }
        .data
        .iter()
        .for_each(|tx| {
            ff.transactions.push(tx.to_owned());
        });
    }

    Ok(())
}

pub(super) async fn load_all_accounts(ff: &mut FFTransactions) -> Result<(), ()> {
    let mut has_more = true;
    let mut page = None;
    while has_more {
        match accounts_api::list_account(&ff.config, None, Some(500), page, None, None).await {
            Ok(accounts) => {
                let pagination = accounts.meta.pagination.clone().unwrap();
                has_more = pagination.current_page < pagination.total_pages;

                let next = pagination.current_page.unwrap() + 1;
                page = Some(next);

                println!(
                    "Page {} of {}",
                    pagination.current_page.unwrap(),
                    pagination.total_pages.unwrap()
                );

                accounts.data
            }
            Err(e) => {
                dbg!(e);
                return Ok(());
            }
        }
        .iter()
        .for_each(|a| ff.accounts.push(a.to_owned()));
    }

    Ok(())
}

pub(super) struct FFTransactions {
    accounts: Vec<AccountRead>,
    transactions: Vec<TransactionRead>,
    config: Configuration,
}

impl FFTransactions {
    pub(super) fn new(config: Configuration) -> Self {
        Self {
            accounts: Vec::with_capacity(1000),
            transactions: Vec::with_capacity(10000),
            config,
        }
    }

    pub(super) fn accounts(&self) -> &Vec<AccountRead> {
        &self.accounts
    }

    pub(super) fn transactions(&self) -> &Vec<TransactionRead> {
        &self.transactions
    }

    pub fn find_account_by_iban(&self, iban: &str) -> Option<&AccountRead> {
        let to_check = Some(Some(iban.to_owned()));

        self.accounts.iter().find(|a| a.attributes.iban == to_check)
    }
}
