mod config;
mod firefly;

use crate::config::AppConfiguration;
use crate::firefly::{load_all_accounts, load_all_transactions, FFTransactions};
use firefly_iii_api::apis::configuration;
use tokio::io::AsyncReadExt;
use gocardless_bankaccount_data_api::models::{JwtObtainPairRequest, StatusEnum};

#[tokio::main]
async fn main() -> Result<(), dotenv::Error> {
    dotenv::dotenv().ok();
    let config = AppConfiguration::from_env()?;

    // let mut ff = FFTransactions::new(configuration::Configuration {
    //     base_path: config.firefly_iii_url,
    //     user_agent: None,
    //     client: Default::default(),
    //     basic_auth: None,
    //     oauth_access_token: None,
    //     bearer_access_token: Some(config.firefly_iii_api_key),
    //     api_key: None,
    // });
    //
    // let _ = load_all_accounts(&mut ff).await;
    // println!("#Accounts:\t{}", ff.accounts().len());

    // let _ = load_all_transactions(&mut ff).await;
    // println!("#Transactions:\t{}", ff.transactions().len());

    let mut gocardless_config = gocardless_bankaccount_data_api::apis::configuration::Configuration::new();
    let gc_token_pair = gocardless_bankaccount_data_api::apis::token_api::obtain_new_access_slash_refresh_token_pair(
        &gocardless_config,JwtObtainPairRequest::new(config.go_cardless_id, config.go_cardless_key)).await.unwrap();

    gocardless_config.bearer_access_token = gc_token_pair.access;


    // let institutions = gocardless_bankaccount_data_api::apis::institutions_api::retrieve_all_supported_institutions_in_a_given_country(
    //     &gocardless_config,
    //     None,
    //     None,
    //     None,
    //     None,
    //     None,
    //     Some("NL"),
    //     None,
    //     None,
    //     None,
    //     None,
    //     None,
    //     None,
    //     None
    // ).await.unwrap();

    let gc_reqs = gocardless_bankaccount_data_api::apis::requisitions_api::retrieve_all_requisitions(&gocardless_config, None, None)
        .await
        .unwrap();

    dbg!("# of requisitions:{} ", &gc_reqs.results.len());


    let active_reqs = gc_reqs
        .results.iter()
        .filter(|req| req.status == Some(StatusEnum::Ln) && req.institution_id != "BUNQ_BUNQNL2A").collect::<Vec<_>>();


    dbg!("# of active requisitions:{} ", &active_reqs.len());

    for req in active_reqs {
        if req.accounts.is_none() {
            dbg!("No active accounts for requisition {}", req.id);
            continue;
        }

        let accounts = req.accounts.as_ref().unwrap();

        for account in accounts {
            let transactions_resp = gocardless_bankaccount_data_api::apis::accounts_api::retrieve_account_transactions(
                &gocardless_config,
                &account.to_string(),
                Some("2024-09-01".to_string()),
                Some("2024-09-30".to_string())
            ).await;

            if transactions_resp.is_err() {
                dbg!("{:?}", transactions_resp.unwrap_err());
                // TODO: Do something smarter here, if possible
                continue;
            }

            let transactions = transactions_resp.unwrap();
            dbg!(&transactions);
        }
    }

    let _ = tokio::io::stdin().read_u8().await;
    Ok(())
}
